@@ -485,6 +485,12 @@ ngx_http_rewrite_return(ngx_conf_t *cf, ngx_command_t *cmd, void *conf)
 
     } else {
 
+        if (ret->status > 999) {
+            ngx_conf_log_error(NGX_LOG_EMERG, cf, 0,
+                               "invalid return code \"%V\"", &value[1]);
+            return NGX_CONF_ERROR;
+        }
+
         if (cf->args->nelts == 2) {
             return NGX_CONF_OK;
         }
@@ -924,7 +930,11 @@ ngx_http_rewrite_set(ngx_conf_t *cf, ngx_command_t *cmd, void *conf)
     if (v->get_handler == NULL
         && ngx_strncasecmp(value[1].data, (u_char *) "http_", 5) != 0
         && ngx_strncasecmp(value[1].data, (u_char *) "sent_http_", 10) != 0
-        && ngx_strncasecmp(value[1].data, (u_char *) "upstream_http_", 14) != 0)
+        && ngx_strncasecmp(value[1].data, (u_char *) "upstream_http_", 14) != 0
+        && ngx_strncasecmp(value[1].data, (u_char *) "cookie_", 7) != 0
+        && ngx_strncasecmp(value[1].data, (u_char *) "upstream_cookie_", 16)
+           != 0
+        && ngx_strncasecmp(value[1].data, (u_char *) "arg_", 4) != 0)
     {
         v->get_handler = ngx_http_rewrite_var;
         v->data = index;
