@@ -606,7 +606,7 @@ ngx_http_perl_ssi(ngx_http_request_t *r, ngx_http_ssi_ctx_t *ssi_ctx,
             return NGX_ERROR;
         }
 
-        asv[0] = (SV *) i;
+        asv[0] = (SV *) (uintptr_t) i;
 
         for (i = 0; args[i]; i++) {
             asv[i + 1] = newSVpvn((char *) args[i]->data, args[i]->len);
@@ -730,7 +730,7 @@ ngx_http_perl_create_interpreter(ngx_conf_t *cf,
 
     n = (pmcf->modules != NGX_CONF_UNSET_PTR) ? pmcf->modules->nelts * 2 : 0;
 
-    embedding = ngx_palloc(cf->pool, (4 + n) * sizeof(char *));
+    embedding = ngx_palloc(cf->pool, (5 + n) * sizeof(char *));
     if (embedding == NULL) {
         goto fail;
     }
@@ -748,6 +748,7 @@ ngx_http_perl_create_interpreter(ngx_conf_t *cf,
     embedding[n++] = "-MNginx";
     embedding[n++] = "-e";
     embedding[n++] = "0";
+    embedding[n] = NULL;
 
     n = perl_parse(my_perl, ngx_http_perl_xs_init, n, embedding, NULL);
 
@@ -846,7 +847,7 @@ ngx_http_perl_call_handler(pTHX_ ngx_http_request_t *r, SV *sub,
     if (args) {
         EXTEND(sp, (intptr_t) args[0]);
 
-        for (i = 1; i <= (ngx_uint_t) args[0]; i++) {
+        for (i = 1; i <= (uintptr_t) args[0]; i++) {
             PUSHs(sv_2mortal(args[i]));
         }
     }
@@ -1620,7 +1621,6 @@ ngx_perl_resolver(SV *name, SV *timeout, SV *cb)
     }
 
     ctx->name    = pr->name;
-    ctx->type    = NGX_RESOLVE_A;
     ctx->handler = ngx_perl_resolver_handler;
     ctx->data    = pr;
     ctx->timeout = clcf->resolver_timeout;
@@ -1722,7 +1722,8 @@ static void
 ngx_perl_resolver_handler(ngx_resolver_ctx_t *ctx)
 {
     ngx_perl_resolver_t  *pr;
-    in_addr_t             addr;
+    ngx_addr_t            addr;
+    u_char               *p;
     ngx_uint_t            i;
     SV                   *cb;
     dSP;
@@ -1767,11 +1768,16 @@ ngx_perl_resolver_handler(ngx_resolver_ctx_t *ctx)
 
         for (i = 0; i < ctx->naddrs; i++) {
             addr = ctx->addrs[i];
-            PUSHs(newSVpvf("%u.%u.%u.%u",
-                           (ntohl(addr) >> 24) & 0xff,
-                           (ntohl(addr) >> 16) & 0xff,
-                           (ntohl(addr) >> 8) & 0xff,
-                           ntohl(addr) & 0xff));
+            // TODO: refactor & INET6
+            switch (addr.sockaddr->sa_family) {
+            case AF_INET:
+                p = (u_char *) &((struct sockaddr_in *) addr.sockaddr)->sin_addr;
+                PUSHs(newSVpvf("%u.%u.%u.%u", p[0], p[1], p[2], p[3]));
+                break;
+            default:
+                PUSHs(newSVpvf("255.255.255.255"));
+                break;
+            }
         }
     }
 
